function [s_p] = EntropicProj(A,b,S0)
%% Entropic Projection function
% This function solves the A*s_p=b problem using the entropic functional
% based cost function. (Iterative search based method)
%INPUT
%A : a 1xN row vector
%S0: a Nx1 column vector
%b : scalar
%OUTPUT
%s_p = Nx1 column Vector


%% Initialize the Program parameters
IterMax = 100;
IterNo = 0;
base =exp(1);
lambda = 2*(b-(A*S0))./(norm(A));  %Initial Lambda Estimate (by L2)

%% Iterations start here
valOld = lambda;
if abs(lambda)>10^-10 
    Region = [-1000,1000];%[lambda-abs(lambda*25),lambda+abs(lambda*25)]; %Initial Lambda search region
    while 1
        try
            [lambdaArray] = constLambdaArray (Region); %Initiate the lambda search array
        catch
            display('err');
            lambda = 0;
            break
        end
        IterNo = IterNo+1;
        temp = repmat(((sign(S0)/log(base)).*(log(abs(S0)+exp(-1))+1))',[length(lambdaArray),1]);
        K = (temp+(lambdaArray'*A/2));
        temp_s=findVal(K,base);
        [val,lambdaindx] = min(abs(b-A*temp_s'));
        if abs(valOld-val)<10^-5
            break
        else
            valOld = val;
            if IterNo>IterMax
                display('Exceeded the maximum number of iterations')
                lambda = lambdaArray(lambdaindx);
                break
            else
                lambda=lambdaArray(lambdaindx);
                if lambdaindx>1
                    if lambdaindx<length(lambdaArray)
                        Region = [lambdaArray(lambdaindx-1),lambdaArray(lambdaindx+1)];
                    else
                        Region = [lambdaArray(lambdaindx-1),lambdaArray(lambdaindx)+(lambdaArray(lambdaindx)-lambdaArray(lambdaindx-1))];
                    end
                else
                    Region = [lambdaArray(lambdaindx)-(lambdaArray(lambdaindx+1)-lambdaArray(lambdaindx)),lambdaArray(lambdaindx+1)];
                end
            end
        end
    end
    K = ((sign(S0)./log(base)).*(log(abs(S0)+exp(-1))+1))+(lambda.*A/2)';
    s_p = findVal(K,base);
else
    s_p = S0;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Supplemantary Functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [s] = sign0(s)
s =sign(s);
s(s==0)=1;

function [s] = findVal(K,base)
% This is for calculating non-exponential version

j = log(base);

s = zeros(size(K));
pindx = find(K*j>0);
nindx = find(K*j<0);
zindx = K==0;
s(pindx) = exp((K(pindx)*j)-1)-exp(-1);
s(nindx) = exp(-1)-exp(-1-(K(nindx)*j));
s(zindx) = 0;

function [s] = findVal_exp(K,base)

j=log(base);

s = zeros(size(K));
pindx = find(K>1);
nindx = find(K<1);
zindx = find(K==1);
s(pindx) = ((K(pindx).^j)-1)/exp(1);
s(nindx) = (1-(K(nindx).^-j))./(exp(1));
s(zindx) = 0;

function[lambdaArray] = constLambdaArray (Region)
const = abs(Region(2)-Region(1))/7;
lambdaArray = Region(1):const:Region(2);