function [Sp] = L1BallProj(b,S0)

%Solves 
%min ||Sp||_1 s.t |S|<=alfa
%S0 = Nx1 column vector (or 2D signals are vectorized)
%b  = 1x1 bound (radius of the ball)

flag = 0;
if size(S0,2)>1
    sz = size(S0);
    S0 = S0(:);
    flag = 1;
end

Sp = ProjectOntoL1Ball(b,S0); %John Duchi's function
if flag == 1
    Sp = reshape(Sp,sz(1),sz(2));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function w = ProjectOntoL1Ball(b,v)
% PROJECTONTOL1BALL Projects point onto L1 ball of specified radius.
%
% w = ProjectOntoL1Ball(v, b) returns the vector w which is the solution
%   to the following constrained minimization problem:
%
%    min   ||w - v||_2
%    s.t.  ||w||_1 <= b.
%
%   That is, performs Euclidean projection of v to the 1-norm ball of radius
%   b.
%
% Author: John Duchi (jduchi@cs.berkeley.edu)

if (b < 0)
  error('Radius of L1 ball is negative: %2.3f\n', b);
end
u = sort(abs(v),'descend');
sv = cumsum(u);
rho = find(u > (sv - b) ./ (1:length(u))', 1, 'last');
theta = max(0, (sv(rho) - b) / rho);
w = sign(v) .* max(abs(v) - theta, 0);