function [s_p] = eppL1Proj(A,b,s0,epp)

%% eppL1 Projection function (sqrt(x.^2-epp))
% This function solves the A*s_p=b problem using the entropic functional
% based cost function. (Iterative search based method)
%INPUT
%A : a 1xN row vector
%s0: a Nx1 column vector
%b : scalar
%epp = epsilon value
%OUTPUT
%s_p = Nx1 column Vector

M = (s0./sqrt(s0.^2+epp))';
pos = A>=0;
neg = ~pos;

poslambdaceil = min((1-M(pos))./A(pos));
poslambdafloor = max((-1-M(pos))./A(pos));
neglambdaceil = max((M(neg)-1)./A(neg));
neglambdafloor = min((M(neg)+1)./A(neg));

if isempty(neglambdaceil)
    neglambdaceil = poslambdaceil;
end
if isempty(neglambdafloor)
    neglambdafloor = poslambdafloor;
end
if isempty(poslambdaceil)
    poslambdaceil = neglambdaceil;
end
if isempty(poslambdafloor)
    poslambdafloor = neglambdafloor;
end

lambdafloor = max(neglambdafloor,poslambdafloor);
lambdaceil = min(neglambdaceil,poslambdaceil);
lambda = (lambdaceil+lambdafloor)/2;

IterMax = 100;
IterNo = 1;
%% Iterations start here
valOld = lambda;
if abs(lambda)>10^-10 || lambda==0
    Region = [lambdafloor,lambdaceil];%[lambda-abs(lambda*25),lambda+abs(lambda*25)]; %Initial Lambda search region
    while 1
        try
            [lambdaArray] = constLambdaArray (Region); %Initiate the lambda search array
        catch
            display('err');
            lambda = 0;
            break
        end
        IterNo = IterNo+1;
        temp = repmat((M),[length(lambdaArray),1]);
        K = (temp+(lambdaArray'*A));
        temp_s=findVal(K,epp);
        
        [val,lambdaindx] = min(abs(b-A*temp_s'));
        if abs(valOld-val)<10^-5
            break
        else
            valOld = val;
            if IterNo>IterMax
                display('Exceeded the maximum number of iterations')
                lambda = lambdaArray(lambdaindx);
                break
            else
                lambda=lambdaArray(lambdaindx);
                if lambdaindx>1
                    if lambdaindx<length(lambdaArray)
                        Region = [lambdaArray(lambdaindx-1),lambdaArray(lambdaindx+1)];
                    else
                        Region = [lambdaArray(lambdaindx-1),lambdaArray(lambdaindx)+(lambdaArray(lambdaindx)-lambdaArray(lambdaindx-1))];
                    end
                else
                    Region = [lambdaArray(lambdaindx)-(lambdaArray(lambdaindx+1)-lambdaArray(lambdaindx)),lambdaArray(lambdaindx+1)];
                end
            end
        end
    end
    K = M+(lambda.*A);
    s_p = findVal(K,epp);
    s_p = s_p';
else
    s_p = s0;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Supplemantary Functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [s] = sign0(s)
s =sign(s);
s(s==0)=1;

function [s] = findVal(K,epp)
% This is for calculating non-exponential version
s = zeros(size(K));
indx0 = abs(K)<=1;
sgn0 = sign(K);
indx_g = ~indx0;
s(indx0) = (sqrt(epp)*K(indx0))./sqrt(1-K(indx0).^2);%sqrt((epp./(1-K(indx0).^2))-epp);
s(indx_g) = 0;
%s = s.*sgn0; %i left the code here

function[lambdaArray] = constLambdaArray (Region)
const = abs(Region(2)-Region(1))/7;
lambdaArray = Region(1):const:Region(2);